// origin source from:
// https://www.icp.uni-stuttgart.de/~hilfer/lehre/100-online/skriptum/html_book00/node76.html

/*********************************************************************
  zupf.cc -- Modellierung einer gezupften Saite
  Autor    : R. Hilfer
  Zweck    : Beispiel zur Vorlesung, demonstriert Karplus-Strong Alg
  Gebrauch : Eingabe von stdin:
               frq = Frequenz in Hertz
               t_max = Gesamtzeit
   Version  : 1.0 (Originalversion) erstellt am  12.10.2000
  -- output = Textfile, then plot using gnuplot
  --- ------------------------------------------------------------
   modified for output on soundcard (linux)           myr 1.4.2014
   g++ -Wall -o zupf zupf.cpp
   usage: 
   ./zupf      then input from stdin
   or
   ./zupf frequency[Hz] duration[sec]      e.g. ./zupf 440 5
   -- output = soundcard
  --- ------------------------------------------------------------
  renamed to propPluckedString.c                     myr 16.5.2019
  modified for running on Propeller Demo Board  ---- Audio Jack
  C++ -> C  change 'new' operator to malloc / free
  remove the stuff for soundcard
  exchange the double array to short array
  compilied with SimpleIDE  C/Memorymodel LMM                     
  -- output = DAC on Prop Demoboard
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h> 
#include <propeller.h>


#define P10 1<<10
#define CRTAMODE (1<<28) + (1<<27) + 10  // DUTY SINGLE ENDED
#define P11 1<<11
#define CRTBMODE (1<<28) + (1<<27) + 11  // DUTY SINGLE ENDED
#define u_0  28000
#define GAIN 12   //max 14


#define STACKSIZE 100
static int cogstack[STACKSIZE];

void pluck (void);
short init (short frq);
void wait (int ticks);
void start_lmm(void *func, int *stack_top );

static	short u[600]; // the ringbuffer for signalforming
static	short len = 0; // the working len
// how many loops through ringbuffer 1,2 or 3
volatile  short d = 2;
// minimal time between samples go to DAC
volatile int sample_duration = 22;
int main (void)
{
  short i;
  start_lmm(pluck, cogstack + 1*STACKSIZE );
L0:
// sounds like normal guitar
  sample_duration = 22; d=2; 
  len = init(262);wait(50000000);
  len = init(294);wait(50000000);
  len = init(330);wait(50000000);
  len = init(350);wait(50000000);
  len = init(393);wait(50000000);
  len = init(440);wait(50000000);
  len = init(495);wait(50000000);
  len = init(524);wait(50000000);
// more synthy  
  sample_duration = 11; d=1;
  len = init(495);wait(50000000);
  len = init(440);wait(50000000);
  len = init(393);wait(50000000);
  len = init(350);wait(50000000);
  len = init(330);wait(50000000);
  len = init(294);wait(50000000);
  len = init(262);wait(50000000); 
// also synthy 
  sample_duration = 11; d=3; 
  len = init(262);wait(10000000);
  len = init(294);wait(10000000);
  len = init(330);wait(10000000);
  len = init(350);wait(10000000);
  len = init(393);wait(10000000);
  len = init(440);wait(10000000);
  len = init(495);wait(10000000);
  len = init(524);wait(10000000); 
//
  sample_duration = 44; d=1;
  len = init(495);wait(50000000);
  len = init(440);wait(50000000);
  len = init(393);wait(50000000);
  len = init(350);wait(50000000);
  len = init(330);wait(50000000);
  len = init(294);wait(50000000);
  goto L0;
	return 0;
}

void pluck (void)
{
  int i;
  unsigned t;
  short uprv; 
   DIRA |= P10; 
   DIRA |= P11;                           
   CTRA = CRTAMODE;
   CTRB = CRTBMODE;  
   DIRA |= 1<<23;

while(1)
{  
  i=0;
  t = d * len ;
  uprv=u[0];

   while( t > 0 )
   {
     // Lowpass = smoothing average 
     u[i] = (u[i] >> 1) + (uprv >> 1);  
     uprv = u[i];            // z^(-1)
     
     FRQA = 0x80000000 + ((int)u[i] << GAIN);
     FRQB = FRQA;
     if(i==len-1) i = -1;    // ringbuffer
     i++; 						       // next index
     t--;  
     usleep(sample_duration);   // 22 = 1/44100       
   }

   OUTA ^= (1<<23);
}//end while(1)
}//end pluck

short init (short frq)
{
	short plckpt = 0;
	short i;
	
	len = 44100/frq; 
	plckpt = len >> 3;  
	
	// Initialize the String
   for( i=0; i<=plckpt; i++ )
   {     
     u[i]= 2 * u_0 * i/plckpt - u_0;
   }
   for( i = len-1; i > plckpt; i-- )
   {
     u[i]= 2 * u_0 * (len-1-i)/(len-plckpt) - u_0;
   }
   
   return len;
}

void wait(int ticks)
{
  waitcnt(CNT + ticks);
}
void start_lmm(void *func, int *stack_top )
{
    extern unsigned int _load_start_kernel[];

    /* push the code address onto the stack */
    --stack_top;
    *stack_top = (int)func;

    /* now start the kernel */
    cognew(_load_start_kernel, stack_top);
}